@@ -1,14 +1,10 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
-
 """Workbench Interactive Shell using IPython"""
+
 import os, sys
-import re
 import hashlib
 import zerorpc
 import IPython
-from IPython.core.prefilter import PrefilterTransformer
-import functools
+import lz4
 from colorama import Fore as F
 
 try:
@@ -21,6 +17,8 @@ try:
     from . import client_helper
     from . import version
     from . import file_streamer
+    from . import repr_to_str_decorator
+    from . import auto_quote_xform
 
 # Okay this happens when you're running in a debugger so having this is
 # super handy and we'll keep it even though it hurts coverage score.
@@ -28,77 +26,9 @@ except (ImportError,ValueError):
     import client_helper
     import version
     import file_streamer
+    import repr_to_str_decorator
+    import auto_quote_xform
 
-# These little helpers get around IPython wanting to take the
-# __repr__ of string output instead of __str__.
-def repr_to_str_decorator(func):
-    """Decorator method for Workbench methods returning a str"""
-
-    @functools.wraps(func)
-    def wrapper(*args, **kwargs):
-        """Decorator method for Workbench methods returning a str"""
-
-        class ReprToStr(str):
-            """Replaces a class __repr__ with it's string representation"""
-            def __repr__(self):
-                return str(self)
-        return ReprToStr(func(*args, **kwargs))
-    return wrapper
-
-# Helper Classes
-class AutoQuoteTransformer(PrefilterTransformer):
-    """IPython Transformer for commands to use 'auto-quotes'"""
-
-    def register_command_set(self, command_set):
-        """Register all the Workbench commands"""
-        self.command_set = command_set
-
-    def transform(self, line, _continue_prompt):
-        """IPython Transformer for commands to use 'auto-quotes'"""
-
-        # Capture the original line
-        orig_line = line
-
-        # Very conservative logic (but possibly flawed)
-        # 1) Lines with any of these symbols ; , ' " ( ) aren't touched
-        # 2) Need to have more than one token
-        # 3) First token in line must be in the workbench command set
-        # 4) If first token is 'help' than all other tokens are quoted
-        # 5) Otherwise only tokens that are not in any of the namespace are quoted
-
-
-        # 1) Lines with any of these symbols ; , ' " ( ) aren't touched
-        skip_symbols = [';', ',', '\'', '"', '(', ')']
-        if any([sym in line for sym in skip_symbols]):
-            return line
-
-        # Build up token set and info
-        token_list = re.split(' |;|,|(|)|\'|"', line)
-        num_tokens = len(token_list)
-        first_token = token_list[0]
-        token_set = set(token_list)
-        if None in token_set: # In some cases None creeps in
-            token_set.remove(None)
-
-        # 2) Need to have more than one token
-        # 3) First token in line must be in the workbench command set
-        if num_tokens > 1 and first_token in self.command_set:
-
-            # 4) If first token is 'help' than all other tokens are quoted
-            if first_token == 'help':
-                token_set.remove('help')
-                for token in token_set:
-                    line = line.replace(token, '"'+token+'"')
-
-            # 5) Otherwise only tokens that are not in any of the namespace are quoted
-            else: # Not help
-                ns_token_set = set([token for nspace in self.shell.all_ns_refs for token in nspace])
-                for token in token_set:
-                    if token not in ns_token_set:
-                        line = line.replace(token, '"'+token+'"')
-
-        # Return the processed line
-        return line
 
 class WorkbenchShell(object):
     """Workbench CLI using IPython Interactive Shell"""
@@ -112,7 +42,8 @@ class WorkbenchShell(object):
         # Grab server arguments
         self.server_info = client_helper.grab_server_args()
 
-        # Spin up workbench server (this sets the self.workbench object)
+        # Spin up workbench server
+        self.workbench = None
         self.connect(self.server_info)
 
         # Create a user session
@@ -157,6 +88,8 @@ class WorkbenchShell(object):
             sys.exit(1)
 
         # Okay do the real connection
+        if self.workbench:
+            self.workbench.close()
         self.workbench = zerorpc.Client(timeout=300, heartbeat=60)
         self.workbench.connect('tcp://'+server_info['server']+':'+server_info['port'])
         print '\n%s<<< Connected: %s:%s >>>%s' % (F.GREEN, server_info['server'], server_info['port'], F.RESET)
@@ -201,6 +134,24 @@ class WorkbenchShell(object):
                 self.ipshell.push({'md5': self.session.md5})
                 self.ipshell.push({'short_md5': self.session.short_md5})
 
+    def pull_df(self, md5):
+        """Wrapper for the get_dataframe workbench method"""
+        try:
+            _packed_df = self.workbench.get_dataframe(md5)
+            _df = pd.read_msgpack(lz4.loads(_packed_df))
+            return _df
+        except zerorpc.exceptions.RemoteError as e:
+            return repr_to_str_decorator.r_to_s(self._data_not_found)(e)
+
+    def search_samples(self, tag=None):
+        """Wrapper for the list_samples workbench method"""
+        
+        # Fixme: This needs to be improved to handle arbitrary predicates (MongoDB predicates)
+        if not tag:
+            return [item['md5'] for item in self.workbench.list_samples()]
+        else:
+            return [item['md5'] for item in self.workbench.list_samples({'type_tag': tag})]
+
     def work_request(self, worker, md5=None):
         """Wrapper for a work_request to workbench"""
 
@@ -214,7 +165,7 @@ class WorkbenchShell(object):
         try:
             return self.workbench.work_request(worker, md5)
         except zerorpc.exceptions.RemoteError as e:
-            return repr_to_str_decorator(self._data_not_found)(e)
+            return repr_to_str_decorator.r_to_s(self._data_not_found)(e)
 
     def workbench_command(self, command, *args):
         """Wrapper for a command to workbench"""
@@ -226,7 +177,7 @@ class WorkbenchShell(object):
         try:
             return self.workbench[command](*args)
         except zerorpc.exceptions.RemoteError as e:
-            return repr_to_str_decorator(self._data_not_found)(e)
+            return repr_to_str_decorator.r_to_s(self._data_not_found)(e)
 
     def _data_not_found(self, e):
         """Message when you get a DataNotFound exception from the server"""
@@ -251,8 +202,11 @@ class WorkbenchShell(object):
         # for some of the workbench commands
         general = {
             'workbench': self.workbench,
-            'help': repr_to_str_decorator(self.workbench.help),
+            'phelp': help,
+            'help': repr_to_str_decorator.r_to_s(self.workbench.help),
             'load_sample': self.load_sample,
+            'pull_df': self.pull_df,
+            'search_samples': self.search_samples,
             'reconnect': lambda info=self.server_info: self.connect(info),
             'version': self.versions,
             'versions': self.versions,
@@ -295,10 +249,14 @@ class WorkbenchShell(object):
         self.ipshell = IPython.terminal.embed.InteractiveShellEmbed(
             config=cfg, banner1='', exit_msg='\nWorkbench has SuperCowPowers...')
 
-        # Register our transformer
-        auto_quoter = AutoQuoteTransformer(self.ipshell, self.ipshell.prefilter_manager)
+        # Register our transformer, the shell will use this for 'shortcut' commands
+        auto_quoter = auto_quote_xform.AutoQuoteTransformer(self.ipshell, self.ipshell.prefilter_manager)
         auto_quoter.register_command_set(self.command_set)
 
+        # Setting up some Pandas options
+        pd.set_option('display.width', 140)
+        pd.set_option('max_colwidth', 15)
+
         # Start up the shell with our set of workbench commands
         self.ipshell(local_ns=self.command_dict)
 
